package com.ruoyi.project.training.spec.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.training.spec.domain.TStApprove;

/**
 * 专项培训计划申请Mapper接口
 * 
 * @author ly
 * @date 2022-04-18
 */
public interface TStApproveMapper 
{
    /**
     * 查询专项培训计划申请
     * 
     * @param id 专项培训计划申请ID
     * @return 专项培训计划申请
     */
    public TStApprove selectTStApproveById(Long id);

    /**
     * 查询专项培训计划申请列表
     * 
     * @param tStApprove 专项培训计划申请
     * @return 专项培训计划申请集合
     */
    @DataScope(deptAlias = "d")
    public List<TStApprove> selectTStApproveList(TStApprove tStApprove);

    /**
     * 新增专项培训计划申请
     * 
     * @param tStApprove 专项培训计划申请
     * @return 结果
     */
    public int insertTStApprove(TStApprove tStApprove);

    /**
     * 修改专项培训计划申请
     * 
     * @param tStApprove 专项培训计划申请
     * @return 结果
     */
    public int updateTStApprove(TStApprove tStApprove);

    /**
     * 删除专项培训计划申请
     * 
     * @param id 专项培训计划申请ID
     * @return 结果
     */
    public int deleteTStApproveById(Long id);

    /**
     * 批量删除专项培训计划申请
     * 
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTStApproveByIds(Long[] ids);
}
